precision highp float;

uniform sampler2D normalTexture;
uniform sampler2D depthTexture;

uniform mat4 inverseCameraMatrix;
uniform vec4 cameraPosition;
uniform vec3 lightDirection;
uniform vec3 lightIntensity;

in vec2 texCoord;

layout(location = 0) out vec3 fragDiffuse;
layout(location = 1) out vec3 fragSpecular;

vec3 CalculateWorldPos(float _depth)
{
	vec4 position = vec4(texCoord.x * 2.0 - 1.0, texCoord.y * 2.0 - 1.0, _depth * 2.0 - 1.0, 1.0);
	position = inverseCameraMatrix * position;
	position = position / position.w;
	return position.xyz;
}

void main()
{
	vec4 texInput = texture2D(normalTexture, texCoord.xy).rgba;
	vec2 depth = texture2D(depthTexture, texCoord.xy).rg;

	vec3 worldPos = CalculateWorldPos(depth.r).xyz;

	vec3 n = normalize(texInput.xyz);
	float specularHardness = texInput.a;

	vec3 view = normalize((worldPos.xyz - cameraPosition.xyz).xyz);
	vec3 r = reflect(-lightDirection, n);

	fragDiffuse = max(0.0, dot(n, -lightDirection)) * lightIntensity;
	fragSpecular = pow(max(0.0, dot(view, r)), texInput.a) * lightIntensity;
}