precision highp float;

uniform sampler2D normalTexture;
uniform sampler2D depthTexture;

uniform mat4 inverseCameraMatrix;
uniform vec4 cameraPosition;
uniform vec3 lightPosition;
uniform vec3 lightIntensity;
uniform vec3 lightAttenuation; //1 / (x + dy + d^2z)
uniform float radius;
uniform int x;

in vec4 screenPosition;

layout(location = 0) out vec3 fragDiffuse;
layout(location = 1) out vec3 fragSpecular;

vec3 CalculateWorldPos(float _depth, vec2 ndc)
{
	vec4 position = vec4(ndc.x, ndc.y, _depth * 2.0 - 1.0, 1.0);
	position = inverseCameraMatrix * position;
	position = position / position.w;
	return position.xyz;
}

void main()
{
	vec2 ndc = screenPosition.xy / screenPosition.w;
	vec2 texNDC = (ndc.xy + vec2(1.0, 1.0)) / 2.0;

	vec2 depthData = texture2D(depthTexture, texNDC.xy).rg;
	vec3 worldPos = CalculateWorldPos(depthData.r, ndc).xyz;

	if (length(worldPos - lightPosition) < radius)
	{
		vec4 texInput = texture2D(normalTexture, texNDC.xy);
		vec3 n = normalize(texInput.xyz);

		if (n == vec3(0.0, 0.0, 0.0))
		{
			discard;
		}
		else
		{
			vec3 lightDirection = lightPosition.xyz - worldPos.xyz;
			float d = length(lightDirection);
			lightDirection = normalize(lightDirection);

			float att = 1.0 / (lightAttenuation.x + lightAttenuation.y * d + lightAttenuation.z * d * d);

			vec3 view = normalize((worldPos.xyz - cameraPosition.xyz).xyz);
			vec3 r = reflect(lightDirection, n);

			fragDiffuse = max(0.0, dot(n, lightDirection)) * lightIntensity * att;
			fragSpecular += pow(max(0.0, dot(view, r)),  texInput.a) * lightIntensity * att;
			//fragSpecular = vec3(0,0,0);
		}
	}
	else
	{
		discard;
	}
}