precision highp float;

uniform sampler2D normalTexture;
uniform sampler2D depthTexture;

uniform mat4 inverseCameraMatrix;
uniform vec4 cameraPosition;
uniform vec3 lightPosition;
uniform vec3 lightDirection;
uniform vec3 lightIntensity;
uniform vec3 lightAttenuation; //1 / (x + dy + d^2z)
uniform float lightAngle;
uniform float lightAngularAttenuation;

in vec4 screenPosition;

out vec3 fragDiffuse;
out vec3 fragSpecular;

vec3 CalculateWorldPos(float _depth, vec2 ndc)
{
	vec4 position = vec4(ndc.x, ndc.y, _depth * 2.0 - 1.0, 1.0);
	position = inverseCameraMatrix * position;
	position = position / position.w;
	return position.xyz;
}

float linInt(float edge0, float edge1, float x)
{
	return clamp((x - edge0) / (edge1 - edge0), 0.0, 1.0);
}

void main()
{
	vec2 ndc = screenPosition.xy / screenPosition.w;
	vec2 texNDC = (ndc.xy + vec2(1.0, 1.0)) / 2.0;

	vec2 depth = texture2D(depthTexture, texNDC.xy).rg;
	vec3 worldPos = CalculateWorldPos(depth.r, ndc).xyz;

	vec3 l = worldPos - lightPosition;
	vec3 lnormal = normalize(l);
	float angle = dot(lnormal, lightDirection);
	float d = length(l);

	if (angle >= cos(lightAngle)) // Otherwise we are outside the cone
	{
		vec4 texInput = texture2D(normalTexture, texNDC.xy).rgba;
		vec3 n = normalize(texInput.xyz);
		float specularHardness = texInput.a;
		
		float att = 1.0 / (lightAttenuation.x + lightAttenuation.y * d + lightAttenuation.z * d * d);
		float angAtt = 1.0 - linInt((1.0 - lightAngularAttenuation) * lightAngle, lightAngle, acos(angle));

		vec3 view = normalize((worldPos.xyz - cameraPosition.xyz).xyz);
		vec3 r = reflect(-lightDirection, n);

		fragDiffuse = max(0.0, dot(n, -lightDirection)) * lightIntensity * att * angAtt;
		fragSpecular = pow(max(0.0, dot(view, r)),  texInput.a) * lightIntensity * att * angAtt;
	}
	else
		discard;
}