precision highp float;

#include "material.glsl"			// This include is handled in the shaderloader
#line 4 4

in VertexFragment vertexData;		// Input from vertex stage

out vec4 fragColor;					// Shaded color

uniform layout(binding = 0) sampler2D diffuseIllumination;		// Input from illumination stage
uniform layout(binding = 1) sampler2D specularIllumination;		// Input from illumination stage

vec4 material(VertexFragment fragment, vec3 diffuseIntensity, vec3 specularIntensity);		// Prototype for user function

void main()
{
	VertexFragment inVertex = vertexData;
	inVertex.position /= inVertex.position.w;
	inVertex.ndc /= inVertex.ndc.w;
	inVertex.normal = normalize(inVertex.normal);
	inVertex.tangent = normalize(inVertex.tangent);

	vec2 illuminationCoord = (inVertex.ndc.xy + vec2(1.0, 1.0)) / 2.0;
	vec3 diffuseIntensity, specularIntensity;

	if (isShadeless)
	{
		diffuseIntensity = vec3(1.0, 1.0, 1.0);
		specularIntensity = vec3(0.0, 0.0, 0.0);
	}
	else
	{
		diffuseIntensity = texture2D(diffuseIllumination, illuminationCoord).rgb;
		specularIntensity = texture2D(specularIllumination, illuminationCoord).rgb;
	}

	fragColor = material(inVertex, diffuseIntensity, specularIntensity);
	//fragColor.rgb = vec3(1,1,1) * (specularIntensity);
	//fragColor.rgb = diffuseIntensity;
	//fragColor = vec4(diffuseIntensity, 1.0);
	fragColor.a = 1.0;
}