precision highp float;

#include "material.glsl"			// This include is handled in the shaderloader
#include "../OctreeShader/octree_common.glsl"
#line 5 5

in VertexFragment vertexData;		// Input from geometry stage

layout (std430, binding = 3) buffer deferredBuffer
{
	DeferredBufferEntry defEntry[];
};

layout(binding = 1, offset = 0) uniform atomic_uint deferredCounter;

FragmentIllumination fragment(VertexFragment fragment);		// Prototype for user function

void main()
{
	VertexFragment inVertex = vertexData;
	inVertex.position /= inVertex.position.w;
	inVertex.ndc /= inVertex.ndc.w;
	inVertex.normal = normalize(inVertex.normal);

	vec4 normalizedVoxelPosition = cameraMatrixArray[2] * inVertex.position;
	normalizedVoxelPosition.xyz /= normalizedVoxelPosition.w;

	FragmentIllumination fragMaterial = fragment(inVertex);

	uint idx = atomicCounterIncrement(deferredCounter);

	DeferredBufferEntry newEntry;
	newEntry.pos = packSnorm3x10(normalizedVoxelPosition.xyz);
	newEntry.normal = packSnorm4x8(vec4(inVertex.normal.xyz, 0.0)); 
	newEntry.emitting = packUnorm4x8(vec4(fragMaterial.emitting.xyz, 0.0));
	newEntry.diffuse = packUnorm4x8(vec4(fragMaterial.diffuse.xyz, 0.0));
	newEntry.specular = packUnorm4x8(vec4(fragMaterial.specular.xyz, 0.0));
	newEntry.opacityHardness = packUnorm4x8(vec4(1.0, fragMaterial.hardness, 0.0, 0.0));

	defEntry[idx] = newEntry;
	
	discard;
}