precision highp float;

in vec2 colorTexUV;

layout(location = 0) out vec3 diffuseIllumination;
layout(location = 1) out vec3 specularIllumination;

uniform layout(binding=6) sampler2DShadow lightDepth;
uniform layout(binding=10) sampler2D normalTexture;
uniform layout(binding=12) sampler2D depthTexture;

uniform mat4 inverseCameraMatrix;
uniform mat4 lightMatrix;
uniform vec3 lightPosition;
uniform vec3 cameraPosition;
uniform float lightAngle;
uniform vec3 lightDirection;
uniform vec3 lightIntensity;

vec4 CalculateWorldPos(float _depth)
{
	vec4 position = vec4(colorTexUV.x * 2.0 - 1.0, colorTexUV.y * 2.0 - 1.0, _depth * 2.0 - 1.0, 1.0);
	position = inverseCameraMatrix * position;
	position = position / position.w;
	return position;
}

void main()
{
	//Read geometry information from deffered textures
	vec4 fragNormal = texture(normalTexture, colorTexUV.xy).xyzw;
	float fragDepth = texture(depthTexture, colorTexUV.xy).x;

	if (fragNormal.xyz == vec3(0,0,0))
	{
		discard;
		return;
	}

	vec4 fragPosition = CalculateWorldPos(fragDepth);
	gl_FragDepth = fragDepth;

	//Shadow
	vec4 posInLight = lightMatrix * fragPosition;
	posInLight /= posInLight.w;
	posInLight.xyz = (posInLight.xyz + vec3(1,1,1)) / 2.0;
	posInLight.z -= 0.001;

	float shadowFactor = texture(lightDepth, posInLight.xyz).x;

	vec3 l = normalize(fragPosition.xyz - lightPosition);

	if (dot(normalize(lightDirection), l) >= cos(lightAngle))
	{
		float diffuse = max(0, dot(fragNormal.xyz, -l)) * shadowFactor;

		vec3 v = normalize(cameraPosition - fragPosition.xyz);
		float specular = pow(max(0, dot(v, reflect(l, fragNormal.xyz))), fragNormal.a) * shadowFactor;

		float test = lightIntensity * (diffuse + specular);
		
		diffuseIllumination = diffuse * lightIntensity;		
		specularIllumination = specular * lightIntensity;
		//specularIllumination = vec3(0,0,0);
	}
	else
	{
		float amb = 0.0;
		diffuseIllumination = vec3(amb, amb, amb);
		specularIllumination = vec3(0,0,0);
	}
}