precision highp float;

in vec2 colorTexUV;

uniform layout(binding=6) sampler2DShadow lightDepth;

uniform mat4 shadowMatrix;
uniform mat4 invOctreeMatrix;
uniform vec3 lightPosition;
uniform float lightAngle;
uniform vec3 lightDirection;
uniform vec3 lightIntensity;

#include "octree_common.glsl"
uniform layout(binding = 0) sampler3D brickTexture;
layout (std430, binding = 1) buffer linkBuffer
{
	LinkBufferEntry entry[];
};

#include "octree_traverse.glsl" //Has to be included after buffer definition

#line 22 22

void main()
{
	int id = 1 + gl_VertexID * 8 + gl_InstanceID;

	uint pos = entry[id].children;

	if (pos == 0U)
		return;

	vec4 worldPosition = invOctreeMatrix * vec4(unpackSnorm3x10(pos).xyz, 1);
	vec3 l = normalize(worldPosition.xyz - lightPosition);

	if (dot(normalize(lightDirection), l) >= cos(lightAngle))
	{
		vec4 posInLight = shadowMatrix * worldPosition;
		posInLight /= posInLight.w;
		posInLight.xyz = (posInLight.xyz + vec3(1,1,1)) / 2.0;
		posInLight.z -= 0.001;

		float shadowFactor = texture(lightDepth, posInLight.xyz).x;

		vec3 normal = normalize(unpackSnorm4x8(entry[id].xplus).xyz);
		vec3 diffuse = unpackUnorm4x8(entry[id].xminus).xyz;

		vec3 intensity = lightIntensity * shadowFactor * diffuse * max(0.0, dot(normal, -l));
		ivec3 writeIntensity = ivec3(intensity * 65535.0);

		atomicAdd(entry[id].yminus, writeIntensity.r);
		atomicAdd(entry[id].zplus, writeIntensity.g);
		atomicAdd(entry[id].zminus, writeIntensity.b);

		/*entry[id].yminus = uint(diffuse.r * 65535.0);
		entry[id].zplus = uint(diffuse.g * 65535.0);
		entry[id].zminus = uint(diffuse.b * 65535.0);*/
	}
}