#include "octree_common.glsl"

#line 4 4

coherent uniform layout(size4x32, binding = 0) image3D brickTexture;

layout (std430, binding = 1) buffer linkBuffer
{
	LinkBufferEntry entry[];
};

#define F1 0.015625
#define F2 0.03125
#define F4 0.0625
#define F8 0.125

uniform int offset;
uniform int level;

void main()
{
	int id = 1 + gl_VertexID * 8 + gl_InstanceID;
	ivec3 texAddr = addr3D(id);
	uint childId = entry[id].children;	

	if (level == 8) //Lowest level
	{
		
		vec4 data[8];

		for (int i = 0; i < 8; i++)
		{
			float opacity = unpackUnorm4x8(entry[childId + i].yplus).x;
			data[i] = vec4(float(entry[childId + i].yminus) / 65535.0, float(entry[childId + i].zplus) / 65535.0, float(entry[childId + i].zminus) / 65535.0, opacity);
			//data[i] *= opacity;
		}

		//Center
		vec4 val = data[0] + data[1] + data[2] + data[3] + data[4] + data[5] + data[6] + data[7];
		imageStore(brickTexture, texAddr + ivec3(1,1,1), val / 8.0);

		//Sides
		val = data[0] + data[1] + data[2] + data[3];
		imageStore(brickTexture, texAddr + ivec3(0,1,1), val / 8.0);
		val = data[4] + data[5] + data[6] + data[7];
		imageStore(brickTexture, texAddr + ivec3(2,1,1), val / 8.0);

		val = data[0] + data[1] + data[4] + data[5];
		imageStore(brickTexture, texAddr + ivec3(1,0,1), val / 8.0);
		val = data[2] + data[3] + data[6] + data[7];
		imageStore(brickTexture, texAddr + ivec3(1,2,1), val / 8.0);

		val = data[0] + data[2] + data[4] + data[6];
		imageStore(brickTexture, texAddr + ivec3(1,1,0), val / 8.0);
		val = data[1] + data[3] + data[5] + data[7];
		imageStore(brickTexture, texAddr + ivec3(1,1,2), val / 8.0);

		//Edges (appear in 4 bricks -> /8)
		val = data[0] + data[2];
		imageStore(brickTexture, texAddr + ivec3(0,1,0), val / 8.0);
		val = data[4] + data[6];
		imageStore(brickTexture, texAddr + ivec3(2,1,0), val / 8.0);
		val = data[5] + data[7];
		imageStore(brickTexture, texAddr + ivec3(2,1,2), val / 8.0);
		val = data[1] + data[3];
		imageStore(brickTexture, texAddr + ivec3(0,1,2), val / 8.0);

		val = data[2] + data[6];
		imageStore(brickTexture, texAddr + ivec3(1,2,0), val / 8.0);
		val = data[6] + data[7];
		imageStore(brickTexture, texAddr + ivec3(2,2,1), val / 8.0);
		val = data[3] + data[7];
		imageStore(brickTexture, texAddr + ivec3(1,2,2), val / 8.0);
		val = data[2] + data[3];
		imageStore(brickTexture, texAddr + ivec3(0,2,1), val / 8.0);

		val = data[0] + data[4];
		imageStore(brickTexture, texAddr + ivec3(1,0,0), val / 8.0);
		val = data[4] + data[5];
		imageStore(brickTexture, texAddr + ivec3(2,0,1), val / 8.0);
		val = data[1] + data[5];
		imageStore(brickTexture, texAddr + ivec3(1,0,2), val / 8.0);
		val = data[0] + data[1];
		imageStore(brickTexture, texAddr + ivec3(0,0,1), val / 8.0);

		//Corners
		imageStore(brickTexture, texAddr + ivec3(0,0,0), data[0] / 8.0);
		imageStore(brickTexture, texAddr + ivec3(2,0,0), data[4] / 8.0);
		imageStore(brickTexture, texAddr + ivec3(2,0,2), data[5] / 8.0);
		imageStore(brickTexture, texAddr + ivec3(0,0,2), data[1] / 8.0);

		imageStore(brickTexture, texAddr + ivec3(0,2,0), data[2] / 8.0);
		imageStore(brickTexture, texAddr + ivec3(2,2,0), data[6] / 8.0);
		imageStore(brickTexture, texAddr + ivec3(2,2,2), data[7] / 8.0);
		imageStore(brickTexture, texAddr + ivec3(0,2,2), data[3] / 8.0);
	}
	else
	{
		vec4 data[5][5][5];
		ivec3 childTexAddr[8];
		for (int i = 0; i < 8; i++)
			childTexAddr[i] = addr3D(int(childId + i));

		for (int x = 0; x < 5; x++)
			for (int y = 0; y < 5; y++)
				for (int z = 0; z < 5; z++)
					data[x][y][z] = vec4(0,0,0,0);

		if (childId != 0)
		{
			for (int x = 0; x < 3; x++)
				for (int y = 0; y < 3; y++)
					for (int z = 0; z < 3; z++)
						data[x][y][z] = imageLoad(brickTexture, childTexAddr[0] + ivec3(x,y,z));

			for (int x = 0; x < 3; x++)
				for (int y = 0; y < 3; y++)
					for (int z = 0; z < 2; z++)
						data[x][y][z+3] = imageLoad(brickTexture, childTexAddr[1] + ivec3(x,y,z+1));

			for (int x = 0; x < 3; x++)
				for (int y = 0; y < 2; y++)
					for (int z = 0; z < 3; z++)
						data[x][y+3][z] = imageLoad(brickTexture, childTexAddr[2] + ivec3(x,y+1,z));

			for (int x = 0; x < 2; x++)
				for (int y = 0; y < 3; y++)
					for (int z = 0; z < 3; z++)
						data[x+3][y][z] = imageLoad(brickTexture, childTexAddr[4] + ivec3(x+1,y,z));

			for (int x = 0; x < 3; x++)
				for (int y = 0; y < 2; y++)
					for (int z = 0; z < 2; z++)
						data[x][y+3][z+3] = imageLoad(brickTexture, childTexAddr[3] + ivec3(x,y+1,z+1));

			for (int x = 0; x < 2; x++)
				for (int y = 0; y < 2; y++)
					for (int z = 0; z < 3; z++)
						data[x+3][y+3][z] = imageLoad(brickTexture, childTexAddr[6] + ivec3(x+1,y+1,z));

			for (int x = 0; x < 2; x++)
				for (int y = 0; y < 3; y++)
					for (int z = 0; z < 2; z++)
						data[x+3][y][z+3] = imageLoad(brickTexture, childTexAddr[5] + ivec3(x+1,y,z+1));

			for (int x = 0; x < 2; x++)
				for (int y = 0; y < 2; y++)
					for (int z = 0; z < 2; z++)
						data[x+3][y+3][z+3] = imageLoad(brickTexture, childTexAddr[7] + ivec3(x+1,y+1,z+1));
		}


		//Center
		vec4 val =
			  F8 * data[2][2][2] +
			  F4 * data[1][2][2] + F4 * data[2][2][1] + F4 * data[3][2][2] + F4 * data[2][2][3] + F4 * data[2][1][2] + F4 * data[2][3][2] +
			  F2 * data[1][2][1] + F2 * data[1][2][3] + F2 * data[3][2][3] + F2 * data[3][2][1] +
			  F2 * data[2][3][1] + F2 * data[1][3][2] + F2 * data[2][3][3] + F2 * data[3][3][2] +
			  F2 * data[2][1][1] + F2 * data[1][1][2] + F2 * data[2][1][3] + F2 * data[3][1][2] +
			  F1 * data[1][3][1] + F1 * data[1][3][3] + F1 * data[3][3][3] + F1 * data[3][3][1] +
			  F1 * data[1][1][1] + F1 * data[1][1][3] + F1 * data[3][1][3] + F1 * data[3][1][1];
		imageStore(brickTexture, texAddr + ivec3(1,1,1), val);

		//Sides
		val = F8 * data[0][2][2] + F4 * data[0][2][1] + F4 * data[0][1][2] + F4 * data[0][2][3] + F4 * data[0][3][2] + 
			  F2 * data[0][3][1] + F2 * data[0][3][3] + F2 * data[0][1][1] + F2 * data[0][1][3] + 
			  F4 * data[1][2][2] + F2 * data[1][2][1] + F2 * data[1][1][2] + F2 * data[1][2][3] + F2 * data[1][3][2] + 
			  F1 * data[1][3][1] + F1 * data[1][3][3] + F1 * data[1][1][1] + F1 * data[1][1][3];
		imageStore(brickTexture, texAddr + ivec3(0,1,1), val);
		val = //F8 * data[4][2][2] + F4 * data[4][2][1] + F4 * data[4][1][2] + F4 * data[4][2][3] + F4 * data[4][3][2] + 
			  //F2 * data[4][3][1] + F2 * data[4][3][3] + F2 * data[4][1][1] + F2 * data[4][1][3] + 
			  F4 * data[3][2][2] + F2 * data[3][2][1] + F2 * data[3][1][2] + F2 * data[3][2][3] + F2 * data[3][3][2] + 
			  F1 * data[3][3][1] + F1 * data[3][3][3] + F1 * data[3][1][1] + F1 * data[3][1][3];
		imageStore(brickTexture, texAddr + ivec3(2,1,1), val);

		val = F8 * data[2][2][0] + F4 * data[1][2][0] + F4 * data[2][1][0] + F4 * data[3][2][0] + F4 * data[2][3][0] + 
			  F2 * data[1][3][0] + F2 * data[1][1][0] + F2 * data[3][1][0] + F2 * data[3][3][0] + 
			  F4 * data[2][2][1] + F2 * data[1][2][1] + F2 * data[2][1][1] + F2 * data[3][2][1] + F2 * data[2][3][1] + 
			  F1 * data[1][3][1] + F1 * data[1][1][1] + F1 * data[3][1][1] + F1 * data[3][3][1];
		imageStore(brickTexture, texAddr + ivec3(1,1,0), val);
		val = //F8 * data[2][2][4] + F4 * data[1][2][4] + F4 * data[2][1][4] + F4 * data[3][2][4] + F4 * data[2][3][4] + 
			  //F2 * data[1][3][4] + F2 * data[1][1][4] + F2 * data[3][1][4] + F2 * data[3][3][4] + 
			  F4 * data[2][2][3] + F2 * data[1][2][3] + F2 * data[2][1][3] + F2 * data[3][2][3] + F2 * data[2][3][3] + 
			  F1 * data[1][3][3] + F1 * data[1][1][3] + F1 * data[3][1][3] + F1 * data[3][3][3];
		imageStore(brickTexture, texAddr + ivec3(1,1,2), val);

		val = F8 * data[2][0][2] + F4 * data[1][0][2] + F4 * data[2][0][1] + F4 * data[3][0][2] + F4 * data[2][0][3] + 
			  F2 * data[1][0][3] + F2 * data[1][0][1] + F2 * data[3][0][1] + F2 * data[3][0][3] + 
			  F4 * data[2][1][2] + F2 * data[1][1][2] + F2 * data[2][1][1] + F2 * data[3][1][2] + F2 * data[2][1][3] + 
			  F1 * data[1][1][3] + F1 * data[1][1][1] + F1 * data[3][1][1] + F1 * data[3][1][3];
		imageStore(brickTexture, texAddr + ivec3(1,0,1), val);
		val = //F8 * data[2][4][2] + F4 * data[1][4][2] + F4 * data[2][4][1] + F4 * data[3][4][2] + F4 * data[2][4][3] + 
			  //7F2 * data[1][4][3] + F2 * data[1][4][1] + F2 * data[3][4][1] + F2 * data[3][4][3] + 
			  F4 * data[2][3][2] + F2 * data[1][3][2] + F2 * data[2][3][1] + F2 * data[3][3][2] + F2 * data[2][3][3] + 
			  F1 * data[1][3][3] + F1 * data[1][3][1] + F1 * data[3][3][1] + F1 * data[3][3][3];
		imageStore(brickTexture, texAddr + ivec3(1,2,1), val);
		
		//Edges
		val = F4 * data[0][1][0] + F2 * data[1][1][0] + F2 * data[0][1][1] + F1 * data[1][1][1] + 
			  F8 * data[0][2][0] + F4 * data[1][2][0] + F4 * data[0][2][1] + F2 * data[1][2][1] + 
			  F4 * data[0][3][0] + F2 * data[1][3][0] + F2 * data[0][3][1] + F1 * data[1][3][1];
		imageStore(brickTexture, texAddr + ivec3(0,1,0), val);
		val = /*F4 * data[4][1][0] +*/ F2 * data[3][1][0] + /*F2 * data[4][1][1] +*/ F1 * data[3][1][1] + 
			  /*F8 * data[4][2][0] +*/ F4 * data[3][2][0] + /*F4 * data[4][2][1] +*/ F2 * data[3][2][1] + 
			  /*F4 * data[4][3][0] +*/ F2 * data[3][3][0] + /*F2 * data[4][3][1] +*/ F1 * data[3][3][1];
		imageStore(brickTexture, texAddr + ivec3(2,1,0), val);
		val = /*F4 * data[4][1][4] + F2 * data[3][1][4] + F2 * data[4][1][3] +*/ F1 * data[3][1][3] + 
			  /*F8 * data[4][2][4] + F4 * data[3][2][4] + F4 * data[4][2][3] +*/ F2 * data[3][2][3] + 
			  /*F4 * data[4][3][4] + F2 * data[3][3][4] + F2 * data[4][3][3] +*/ F1 * data[3][3][3];
		imageStore(brickTexture, texAddr + ivec3(2,1,2), val);
		val = /*F4 * data[0][1][4] + F2 * data[1][1][4] +*/ F2 * data[0][1][3] + F1 * data[1][1][3] + 
			  /*F8 * data[0][2][4] + F4 * data[1][2][4] +*/ F4 * data[0][2][3] + F2 * data[1][2][3] + 
			  /*F4 * data[0][3][4] + F2 * data[1][3][4] +*/ F2 * data[0][3][3] + F1 * data[1][3][3];
		imageStore(brickTexture, texAddr + ivec3(0,1,2), val);


		val = /*F4 * data[1][4][0] + F2 * data[1][4][1] +*/ F2 * data[1][3][0] + F1 * data[1][3][1] + 
			  /*F8 * data[2][4][0] + F4 * data[2][4][1] +*/ F4 * data[2][3][0] + F2 * data[2][3][1] + 
			  /*F4 * data[3][4][0] + F2 * data[3][4][1] +*/ F2 * data[3][3][0] + F1 * data[3][3][1];
		imageStore(brickTexture, texAddr + ivec3(1,2,0), val);
		val = /*F4 * data[4][4][1] + F2 * data[3][4][1] + F2 * data[4][3][1] +*/ F1 * data[3][3][1] + 
			  /*F8 * data[4][4][2] + F4 * data[3][4][2] + F4 * data[4][3][2] +*/ F2 * data[3][3][2] + 
			  /*F4 * data[4][4][3] + F2 * data[3][4][3] + F2 * data[4][3][3] +*/ F1 * data[3][3][3];
		imageStore(brickTexture, texAddr + ivec3(2,2,1), val);
		val = /*F4 * data[1][4][4] + F2 * data[1][4][3] + F2 * data[1][3][4] +*/ F1 * data[1][3][3] + 
			  /*F8 * data[2][4][4] + F4 * data[2][4][3] + F4 * data[2][3][4] +*/ F2 * data[2][3][3] + 
			  /*F4 * data[3][4][4] + F2 * data[3][4][3] + F2 * data[3][3][4] +*/ F1 * data[3][3][3];
		imageStore(brickTexture, texAddr + ivec3(1,2,2), val);
		val = /*F4 * data[0][4][1] + F2 * data[1][4][1] +*/ F2 * data[0][3][1] + F1 * data[1][3][1] + 
			  /*F8 * data[0][4][2] + F4 * data[1][4][2] +*/ F4 * data[0][3][2] + F2 * data[1][3][2] + 
			  /*F4 * data[0][4][3] + F2 * data[1][4][3] +*/ F2 * data[0][3][3] + F1 * data[1][3][3];
		imageStore(brickTexture, texAddr + ivec3(0,2,1), val);


		val = F4 * data[1][0][0] + F2 * data[1][1][0] + F2 * data[1][0][1] + F1 * data[1][1][1] + 
			  F8 * data[2][0][0] + F4 * data[2][1][0] + F4 * data[2][0][1] + F2 * data[2][1][1] + 
			  F4 * data[3][0][0] + F2 * data[3][1][0] + F2 * data[3][0][1] + F1 * data[3][1][1];
		imageStore(brickTexture, texAddr + ivec3(1,0,0), val);
		val = /*F4 * data[4][0][1] + F2 * data[4][1][1] +*/ F2 * data[3][0][1] + F1 * data[3][1][1] + 
			  /*F8 * data[4][0][2] + F4 * data[4][1][2] +*/ F4 * data[3][0][2] + F2 * data[3][1][2] + 
			  /*F4 * data[4][0][3] + F2 * data[4][1][3] +*/ F2 * data[3][0][2] + F1 * data[3][1][3];
		imageStore(brickTexture, texAddr + ivec3(2,0,1), val);
		val = /*F4 * data[1][0][4] + F2 * data[1][1][4] +*/ F2 * data[1][0][3] + F1 * data[1][1][3] + 
			  /*F8 * data[2][0][4] + F4 * data[2][1][4] +*/ F4 * data[2][0][3] + F2 * data[2][1][3] + 
			  /*F4 * data[3][0][4] + F2 * data[3][1][4] +*/ F2 * data[3][0][3] + F1 * data[3][1][3];
		imageStore(brickTexture, texAddr + ivec3(1,0,2), val);
		val = F4 * data[0][0][1] + F2 * data[0][1][1] + F2 * data[1][0][1] + F1 * data[1][1][1] + 
			  F8 * data[0][0][2] + F4 * data[0][1][2] + F4 * data[1][0][2] + F2 * data[1][1][2] + 
			  F4 * data[0][0][3] + F2 * data[0][1][3] + F2 * data[1][0][3] + F1 * data[1][1][3];
		imageStore(brickTexture, texAddr + ivec3(0,0,1), val);

		//Corners
		val = F8 * data[0][0][0] + F4 * data[0][1][0] + F2 * data[1][1][0] + F4 * data[1][0][0] + F4 * data[0][0][1] + F2 * data[1][0][1] + F1 * data[1][1][1] + F2 * data[0][1][1];
		imageStore(brickTexture, texAddr + ivec3(0,0,0), val);
		val = /*F8 * data[4][0][0] +*/ F4 * data[3][0][0] + F2 * data[3][1][0] + /*F4 * data[4][1][0] + F4 * data[4][0][1] +*/ F2 * data[3][0][1] + F1 * data[3][1][1];// + F2 * data[4][1][1];
		imageStore(brickTexture, texAddr + ivec3(2,0,0), val);
		val = /*F8 * data[4][0][4] + F4 * data[3][0][4] + F2 * data[3][1][4] + F4 * data[4][1][4] + F4 * data[4][0][3] +*/ F2 * data[3][0][3] + F1 * data[3][1][3];// + F2 * data[4][1][3];
		imageStore(brickTexture, texAddr + ivec3(2,0,2), val);
		val = /*F8 * data[0][0][4] +*/ F4 * data[1][0][4] + /*F2 * data[1][1][4] + F4 * data[0][1][4] +*/ F4 * data[0][0][3] + F2 * data[1][0][3] + F1 * data[1][1][3] + F2 * data[0][1][3];
		imageStore(brickTexture, texAddr + ivec3(0,0,2), val);

		val = /*F8 * data[0][4][0] + F4 * data[1][4][0] + F2 * data[1][4][1] + F4 * data[0][4][1] +*/ F4 * data[0][3][0] + F2 * data[1][3][0] + F1 * data[1][3][1] + F2 * data[0][3][1];
		imageStore(brickTexture, texAddr + ivec3(0,2,0), val);
		val = /*F8 * data[4][4][0] + F4 * data[3][4][0] +*/ F2 * data[3][3][0] + /*F4 * data[4][3][0] + F4 * data[4][4][1] + F2 * data[3][4][1] +*/ F1 * data[3][3][1];// + F2 * data[4][3][1];
		imageStore(brickTexture, texAddr + ivec3(2,2,0), val);
		val = /*F8 * data[4][4][4] + F4 * data[3][4][4] + F2 * data[3][3][4] + F4 * data[4][3][4] + F4 * data[4][4][3] + F2 * data[3][4][3] + */F1 * data[3][3][3];// + F2 * data[4][3][3];
		imageStore(brickTexture, texAddr + ivec3(2,2,2), val);
		val = /*F8 * data[0][4][4] + F4 * data[1][4][4] + F2 * data[1][3][4] + F4 * data[0][3][4] + F4 * data[0][4][3] + F2 * data[1][4][3] +*/ F1 * data[1][3][3] + F2 * data[0][3][3];
		imageStore(brickTexture, texAddr + ivec3(0,2,2), val);
	}
}