#include "octree_common.glsl"

#line 3 3

coherent uniform layout(size4x32, binding = 0) image3D brickTexture;

layout (std430, binding = 1) buffer linkBuffer
{
	LinkBufferEntry entry[];
};

uniform int direction;

void main()
{
	int id = 1 + gl_VertexID * 8 + gl_InstanceID;
	LinkBufferEntry e = entry[id];

	ivec3 texAddr = addr3D(id);

	if (direction == 0) // X-Axis
	{
		ivec3 neighborAddr = addr3D(int(e.xplus));

		for (int i = 0; i < 3; i++)
		{
			for (int j = 0; j < 3; j++)
			{
				vec4 op1 = imageLoad(brickTexture, texAddr + ivec3(2,i,j)) * sign(id);
				vec4 op2 = imageLoad(brickTexture, neighborAddr + ivec3(0,i,j)) * sign(e.xplus);

				op1 = (op1 + op2) * sign(e.xplus);

				imageStore(brickTexture, texAddr + ivec3(2,i,j), op1);
				imageStore(brickTexture, neighborAddr + ivec3(0,i,j), op1);

				if (e.xminus == 0)
					imageStore(brickTexture, texAddr + ivec3(0,i,j), vec4(0,0,0,0));
			}
		}
	}
	else if (direction == 1)
	{
		ivec3 neighborAddr = addr3D(int(e.yplus));

		for (int i = 0; i < 3; i++)
		{
			for (int j = 0; j < 3; j++)
			{
				vec4 op1 = imageLoad(brickTexture, texAddr + ivec3(i,2,j)) * sign(id);
				vec4 op2 = imageLoad(brickTexture, neighborAddr + ivec3(i,0,j)) * sign(e.yplus);

				op1 = (op1 + op2) * sign(e.yplus);

				imageStore(brickTexture, texAddr + ivec3(i,2,j), op1);
				imageStore(brickTexture, neighborAddr + ivec3(i,0,j), op1);

				if (e.yminus == 0)
					imageStore(brickTexture, texAddr + ivec3(i,0,j), vec4(0,0,0,0));
			}
		}
	}
	else if (direction == 2)
	{
		ivec3 neighborAddr = addr3D(int(e.zplus));

		for (int i = 0; i < 3; i++)
		{
			for (int j = 0; j < 3; j++)
			{
				vec4 op1 = imageLoad(brickTexture, texAddr + ivec3(i,j,2)) * sign(id);
				vec4 op2 = imageLoad(brickTexture, neighborAddr + ivec3(i,j,0)) * sign(e.zplus);

				op1 = (op1 + op2) * sign(e.zplus);

				imageStore(brickTexture, texAddr + ivec3(i,j,2), op1);
				imageStore(brickTexture, neighborAddr + ivec3(i,j,0), op1);

				if (e.zminus == 0)
					imageStore(brickTexture, texAddr + ivec3(i,j,0), vec4(0,0,0,0));
			}
		}
	}
}