#include "octree_common.glsl"

#line 3 3

uniform mat4 cameraMatrix;
uniform mat4 invGIMatrix;
uniform uint level;
uniform uint leafLevel;

layout(points) in;
layout(line_strip, max_vertices = 24) out;

in int id[];

out vec3 color;
out vec3 texCoord;
out flat int fragId;

coherent uniform layout(size1x32, binding = 2) uimageBuffer lockBuffer;

layout (std430, binding = 1) buffer linkBuffer
{
	LinkBufferEntry entry[];
};

void main()
{
	mat4 totMatrix = cameraMatrix * invGIMatrix;
	int thisId = id[0];
	fragId = thisId;

	uint lock = imageLoad(lockBuffer, thisId).x;
	if (lock < 2U)
		return;

	vec3 pos;
	float halfVoxel = 1.0 / pow(2, level);

	if (level == leafLevel)
	{
		pos = unpackSnorm3x10(entry[thisId].children);

		ivec3 lightIntensityInt = ivec3(entry[thisId].yminus, entry[thisId].zplus, entry[thisId].zminus);
		vec3 lightIntensity = vec3(lightIntensityInt) / 65535.0;

		//color = unpackUnorm4x8(entry[thisId].xminus).xyz;
		color = lightIntensity;
	}
	else //Move down the tree and try to find a leaf voxel
	{
		color = vec3(0,0,1);

		uint idx = entry[thisId].children;
		vec3 correctVector = vec3(0,0,0);
		vec3 size = vec3(1,1,1);
		size /= 2.0;

		for (uint i = level; i < leafLevel; i++)
		{
			size /= 2.0;
			
			for (uint off = 0; off < 8; off++)
			{
				lock = imageLoad(lockBuffer, int(idx + off));
				
				if (lock == 2U)
				{
					idx = entry[idx + off].children;
					
					ivec3 dir = ivec3(off / 4, 0, 0);
					dir.y = (int(off) - dir.x * 4) / 2;
					dir.z = int(off) - dir.x * 4 - dir.y * 2;
					
					dir = dir * 2 - ivec3(1,1,1);

					if (i < leafLevel)
						correctVector = correctVector +  size * -1.0 * vec3(dir);

					break;
				}
			}
		}

		pos = unpackSnorm3x10(idx) + correctVector * 2.0 * halfVoxel;
	}

	halfVoxel *= 0.95;

	gl_Position = totMatrix * vec4(pos.x - halfVoxel, pos.y - halfVoxel, pos.z - halfVoxel, 1.0);
	texCoord = vec3(0,0,0);
	EmitVertex();
	gl_Position = totMatrix * vec4(pos.x + halfVoxel, pos.y - halfVoxel, pos.z - halfVoxel, 1.0);
	texCoord = vec3(1,0,0);
	EmitVertex();
	EndPrimitive();
	gl_Position = totMatrix * vec4(pos.x + halfVoxel, pos.y - halfVoxel, pos.z - halfVoxel, 1.0);
	texCoord = vec3(1,0,0);
	EmitVertex();
	gl_Position = totMatrix * vec4(pos.x + halfVoxel, pos.y + halfVoxel, pos.z - halfVoxel, 1.0);
	texCoord = vec3(1,1,0);
	EmitVertex();
	EndPrimitive();
	gl_Position = totMatrix * vec4(pos.x + halfVoxel, pos.y + halfVoxel, pos.z - halfVoxel, 1.0);
	texCoord = vec3(1,1,0);
	EmitVertex();
	gl_Position = totMatrix * vec4(pos.x - halfVoxel, pos.y + halfVoxel, pos.z - halfVoxel, 1.0);
	texCoord = vec3(0,1,0);
	EmitVertex();
	EndPrimitive();
	gl_Position = totMatrix * vec4(pos.x - halfVoxel, pos.y + halfVoxel, pos.z - halfVoxel, 1.0);
	texCoord = vec3(0,1,0);
	EmitVertex();
	gl_Position = totMatrix * vec4(pos.x - halfVoxel, pos.y - halfVoxel, pos.z - halfVoxel, 1.0);
	texCoord = vec3(0,0,0);
	EmitVertex();
	EndPrimitive();

	gl_Position = totMatrix * vec4(pos.x - halfVoxel, pos.y - halfVoxel, pos.z + halfVoxel, 1.0);
	texCoord = vec3(0,0,1);
	EmitVertex();
	gl_Position = totMatrix * vec4(pos.x + halfVoxel, pos.y - halfVoxel, pos.z + halfVoxel, 1.0);
	texCoord = vec3(1,0,1);
	EmitVertex();
	EndPrimitive();
	gl_Position = totMatrix * vec4(pos.x + halfVoxel, pos.y - halfVoxel, pos.z + halfVoxel, 1.0);
	texCoord = vec3(1,0,1);
	EmitVertex();
	gl_Position = totMatrix * vec4(pos.x + halfVoxel, pos.y + halfVoxel, pos.z + halfVoxel, 1.0);
	texCoord = vec3(1,1,1);
	EmitVertex();
	EndPrimitive();
	gl_Position = totMatrix * vec4(pos.x + halfVoxel, pos.y + halfVoxel, pos.z + halfVoxel, 1.0);
	texCoord = vec3(1,1,1);
	EmitVertex();
	gl_Position = totMatrix * vec4(pos.x - halfVoxel, pos.y + halfVoxel, pos.z + halfVoxel, 1.0);
	texCoord = vec3(0,1,1);
	EmitVertex();
	EndPrimitive();
	gl_Position = totMatrix * vec4(pos.x - halfVoxel, pos.y + halfVoxel, pos.z + halfVoxel, 1.0);
	texCoord = vec3(0,1,1);
	EmitVertex();
	gl_Position = totMatrix * vec4(pos.x - halfVoxel, pos.y - halfVoxel, pos.z + halfVoxel, 1.0);
	texCoord = vec3(0,0,1);
	EmitVertex();
	EndPrimitive();

	gl_Position = totMatrix * vec4(pos.x - halfVoxel, pos.y - halfVoxel, pos.z - halfVoxel, 1.0);
	texCoord = vec3(0,0,0);
	EmitVertex();
	gl_Position = totMatrix * vec4(pos.x - halfVoxel, pos.y - halfVoxel, pos.z + halfVoxel, 1.0);
	texCoord = vec3(0,0,1);
	EmitVertex();
	EndPrimitive();

	gl_Position = totMatrix * vec4(pos.x + halfVoxel, pos.y - halfVoxel, pos.z - halfVoxel, 1.0);
	texCoord = vec3(1,0,0);
	EmitVertex();
	gl_Position = totMatrix * vec4(pos.x + halfVoxel, pos.y - halfVoxel, pos.z + halfVoxel, 1.0);
	texCoord = vec3(1,0,1);
	EmitVertex();
	EndPrimitive();

	gl_Position = totMatrix * vec4(pos.x + halfVoxel, pos.y + halfVoxel, pos.z - halfVoxel, 1.0);
	texCoord = vec3(1,1,0);
	EmitVertex();
	gl_Position = totMatrix * vec4(pos.x + halfVoxel, pos.y + halfVoxel, pos.z + halfVoxel, 1.0);
	texCoord = vec3(1,1,1);
	EmitVertex();
	EndPrimitive();

	gl_Position = totMatrix * vec4(pos.x - halfVoxel, pos.y + halfVoxel, pos.z - halfVoxel, 1.0);
	texCoord = vec3(0,1,0);
	EmitVertex();
	gl_Position = totMatrix * vec4(pos.x - halfVoxel, pos.y + halfVoxel, pos.z + halfVoxel, 1.0);
	texCoord = vec3(0,1,1);
	EmitVertex();
	EndPrimitive();
	
}