#line 0 0

struct octStackEntry
{
	uint addr;
	vec3 center;
};

vec3 lowestSize;

octStackEntry octStack[12];
int stackFill = 0;

vec3 posInBrick(vec3 _pos, uint _idx, vec3 _center, vec3 _halfsize)
{
	ivec3 texSize = textureSize(brickTexture, 0);
	vec3 texelSize = 1.0 / vec3(texSize);

	vec3 offset = (_pos - _center + _halfsize) / _halfsize;
	offset += 0.5;

	vec3 coord = (addr3D(int(_idx)) + offset) * texelSize;

	return coord;
}

bool traverseLevel(vec3 _position, uint _address, inout vec3 _center, inout vec3 _halfSize, out uint _childAddress)
{
	uint childAddr = entry[_address].children;

	if (childAddr == 0U)
		return false;

	vec3 octdir = _position - _center;
	ivec3 octvec = max(ivec3(0,0,0), ivec3(sign(octdir)));
	int octidx = octvec.x * 4 + octvec.y * 2 + octvec.z;

	_childAddress = childAddr + octidx;

	_halfSize /= 2.0;
	_center += _halfSize * sign(octdir);
	return true;
}

bool traverseTopDown(vec3 _position, int _level, out vec3 _center, out vec3 _halfSize, out uint _childAddress)
{
	octStack[0] = octStackEntry(0, vec3(0,0,0));
	lowestSize = vec3(1,1,1);
	stackFill = 0;
	_halfSize = vec3(1,1,1);
	_center = vec3(0,0,0);
	
	_childAddress = 0;

	for (int lvl = 0; lvl < _level; lvl++)
	{
		uint childAddr;
		if (!traverseLevel(_position, _childAddress, _center, _halfSize, childAddr))
			return false;

		//Write Information to Stack
		stackFill++;
		octStack[stackFill] = octStackEntry(childAddr, _center);
		lowestSize = _halfSize;

		_childAddress = childAddr;
	}	

	return true;
}

bool traverseStack(vec3 _position, int _level, out vec3 _center, out vec3 _halfSize, out uint _childAddress)
{
	//Read information from lowest level that can be found
	int low = min(_level, stackFill);

	//Search stack up until we find box
	float s = 1.0 / pow(2.0, low);
	for (stackFill = low; stackFill > 0; stackFill--)
	{
		vec3 relVec = abs(_position - octStack[stackFill].center);
		if (relVec.x < s && relVec.y < s && relVec.z < s)
			break;
		s *= 2.0;
	}
	_halfSize = vec3(s,s,s);
	_center = octStack[stackFill].center;
	_childAddress = octStack[stackFill].addr;

	for (int lvl = stackFill; lvl < _level; lvl++)
	{
		uint childAddr;
		if (!traverseLevel(_position, _childAddress, _center, _halfSize, childAddr))
			return false;

		//Write Information to Stack
		stackFill++;
		octStack[stackFill] = octStackEntry(childAddr, _center);

		_childAddress = childAddr;
	}	
} 

bool traverse(vec3 _position, int _level, out vec3 _center, out vec3 _halfSize, out uint _childAddress)
{
	return traverseTopDown(_position, _level, _center, _halfSize, _childAddress);
}

#line 0 0